/*******************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized. This
 * software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
 * LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
 * ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software
 * and to discontinue the availability of this software. By using this software,
 * you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer*
 * Copyright (C) 2014 Renesas Electronics Corporation. All rights reserved.
 *******************************************************************************/
/**************************************************************************//**
* @file         main.c
* $Rev: 1236 $
* $Date:: 2014-10-27 16:03:38 +0900#$
* @brief        BSP main Display Sample
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdio.h>
#include "cmsis_os.h"

#include "Serial.h"

#include "r_errno.h"
#include "ioif_public.h"
#include "errno2str.h"

#include "dma_if.h"
#include "riic_if.h"


#include "display_if.h"
#include "displaywrapper_if.h"
#include "lcd_if.h"
#include "bsp_util.h"

/******************************************************************************
Macro definitions
******************************************************************************/
#define USE_LCD     0 /* 1:Initialize LCD, 0:Skip LCD */

/* setup RIIC Channel */
#if (LCD_BOARD == LCD_BOARD_LCDKITB01)

#if (LCD_KITB01_CH == LCD_KITB01_CH_0)
#define RIIC_CH     0
#elif (LCD_KITB01_CH == LCD_KITB01_CH_1)
#define RIIC_CH     3
#else
#error LCD_KITB01_CH is incorrect setting
#endif /* LCD_KITB01_CH */

#elif (LCD_BOARD == LCD_BOARD_TITSC2007)
#define RIIC_CH     3

#else
#error LCD_BOARD is incorrect setting
#endif /* LCD_BOARD */

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/*----------------------------------------------------------------------------
 *   IOIF Global:
 *---------------------------------------------------------------------------*/

static IOIF_CONFIG ioif_cfg;

/*----------------------------------------------------------------------------
 *   DMA Global:
 *---------------------------------------------------------------------------*/

static dma_drv_init_t dma_init_param;
void userdef_dma_error_callback(union sigval signo);
static AIOCB dma_err_aio;

/*----------------------------------------------------------------------------
 *   RIIC Global:
 *---------------------------------------------------------------------------*/

static riic_channel_cfg_t riic_cfg[RIIC_CH_NUM];

/**************************************************************************//**
* Function Name: main
* @brief         CMSIS RTOS Task : Main task
*
*                Description:<br>
*                Application initialize task
* @param         none
* @retval        none
******************************************************************************/
int_t main(void)
{
    int32_t ret;
    int_t i;

    /***********************************************************************/
    /* Initialise Debug UART */
    /***********************************************************************/

    SER_Init();

    /***********************************************************************/
    /* Print Board Information */
    /***********************************************************************/

    R_BSP_PrintBootMode();

    /***********************************************************************/
    /* Initialise IOIF */
    /***********************************************************************/
    ioif_cfg.max_devices       = 10;
    ioif_cfg.max_drivers       = 10;
    ioif_cfg.max_open_files    = 10;
    ioif_cfg.taskPriority      = osPriorityNormal;

    ret = ioif_initialise(&ioif_cfg);
    printf("ioif_initialise return %d\n", ret);
    if (EERROR == ret)
    {
        printf(" LastError = %d\n", *ioif_get_errno());
    }

    /***********************************************************************/
    /* Initialise DMA */
    /***********************************************************************/

    for (i = 0; i < DMA_CH_NUM; i++)
    {
        dma_init_param.channel[i] = true;
    }

    dma_err_aio.aio_sigevent.sigev_notify = SIGEV_THREAD;
    dma_err_aio.aio_sigevent.sigev_notify_function = &userdef_dma_error_callback;
    dma_init_param.p_aio = &dma_err_aio;

    ret = R_DMA_Init(&dma_init_param, NULL);
    printf("R_DMA_Init return %d\n", ret);

    /***********************************************************************/
    /* Initialise RIIC */
    /***********************************************************************/

    for (i = 0; i < RIIC_CH_NUM; i++)
    {
        riic_cfg[i].enabled    = false;
    }

    riic_cfg[RIIC_CH].enabled                   = true;
    riic_cfg[RIIC_CH].int_level                 = 0x78;
    riic_cfg[RIIC_CH].task_pri                  = osPriorityNormal;
    riic_cfg[RIIC_CH].scl_timeout               = RIIC_TIMEOUT_16;
    riic_cfg[RIIC_CH].enable_scl_add_output     = false;
    riic_cfg[RIIC_CH].select_dma                = RIIC_COPY_MODE_PIO;
    riic_cfg[RIIC_CH].addr_mode_set.trans_mode  = RIIC_MODE_MASTER;
    riic_cfg[RIIC_CH].addr_mode_set.device_addr = 0x84>>1;

    /* SCL clock 50kbps */
    riic_cfg[RIIC_CH].scl_ctl.div_rate          = RIIC_PCLK_DIV_16;
    riic_cfg[RIIC_CH].scl_ctl.high_width        = 17;
    riic_cfg[RIIC_CH].scl_ctl.low_width         = 20;

    riic_cfg[RIIC_CH].scl_ctl.enable_filter     = false;

    ret = ioif_start_device(R_RIIC_MakeCbTbl(), &riic_cfg, "\\dev\\riic", "", NULL);
    printf("ioif_start_device return %d\n", ret);
    if (EERROR == ret)
    {
        printf(" LastError = %d, %s\n", *ioif_get_errno(), errno2str(*ioif_get_errno()));
    }

    /***********************************************************************/
    /* Initialise LCD Board */
    /***********************************************************************/

#if USE_LCD
    /* LCD setup */
    LCD_Init();
    LCD_Open( 8, osPriorityAboveNormal, 1024 );
    /* Backlight ON */
    LCD_SetBacklight(0);
    osDelay(1000);
    LCD_SetBacklight(100);
#endif

    /***********************************************************************/
    /* Start Display Demo */
    /***********************************************************************/
    printf("Display Driver Version:0x%02x\n", (uint32_t)R_DISPLAY_GetVersion());

#if 1       /* You are switched to display operation. */
    printf("_/_/_/_/  Graphics Sample  _/_/_/_/\n\n");      /*Sample to show the graphic*/
    Graphics_GraphicsSample();

#else
    printf("_/_/_/_/  Video Sample  _/_/_/_/\n\n");         /*Sample to show the video*/
    Graphics_VideoSample();
#endif

#if USE_LCD
    /* Backlight OFF */
    LCD_SetBacklight(0);
    LCD_Close();
#endif
    ret = ioif_uninitialise();
    if (EERROR == ret)
    {
        printf(" LastError = %d, %s\n", *ioif_get_errno(), errno2str(*ioif_get_errno()));
    }
    osThreadTerminate(osThreadGetId());

    return 0;
}
/**************************************************************************//**
* Function Name: userdef_dma_error_callback
* @brief         DMA driver : error callback function
*
*                Description:<br>
*                DMA error interrupt callback function
* @param[in]     signo.sival_int : bitmap of error channel(bit0=ch0)
* @retval        none
******************************************************************************/
void userdef_dma_error_callback(union sigval signo)
{
    printf("DMAERR MAP:0x%02x\n", signo.sival_int);
}

