/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2017 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: jcu_mt.c
*    JPEG Codec Unit (JCU) driver API. Multi Thread Support Code.
*
* - $Version: $ (=JCU_VERSION)
* - $ModuleRevision: {Masters}\JCU\RZ\48_RStyle2 $
* - $Rev: $
* - $Date: 2017-12-25T18:37:17+09:00 $
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "use_list.h"
#include  "r_jcu_user.h"
#include  "r_jcu_api.h"
#include  "../src/r_jcu_local.h"
#include  "r_ospl.h"
#include  "r_ospl_os_less.h"  /* For portability */
#ifdef  USE_LIST_FREE_RTOS
#include  "FreeRTOS.h"
#include  "task.h"
#endif

#if  JCU_MULTI_THREAD


/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Define: GS_IS_ALWAYS_SET_ENCODE_TABLES
*    0 or 1.
************************************************************************/
#define  GS_IS_ALWAYS_SET_ENCODE_TABLES  1

#if  GS_IS_ALWAYS_SET_ENCODE_TABLES == 0
#include  "iodefine.h"
#endif


/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Structure: jcu_contexts_t
*    Contexts of threads.
************************************************************************/
typedef struct
{
    r_ospl_table_t*  ThreadMemory;
    jcu_thread_t*    ThreadMemoryArray;
    jcu_context_t*   CurrentContextInInterrupt;
} jcu_contexts_t;


/***********************************************************************
* Structure: jcu_operator_t
*    JCU operator for multi thread. Internal thread context.
************************************************************************/
#if  JCU_MULTI_THREAD
typedef struct
{
    r_ospl_async_state_t    State;
    r_ospl_thread_id_t      ThreadID;
    r_ospl_event_flags_t    MultiThreadEvent;
    r_ospl_queue_id_t       MultiThreadQueue;
    r_ospl_event_flags_t    ExitEvent;
    r_ospl_async_t          FinalAsync;
    jcu_context_t*          Owner[ F_JCU_MT_CountOfDataBits ];
    jcu_operator_config_t*  Config;

#if ! R_OSPL_IS_PREEMPTION
    int_fast32_t            StepIndex;

    /* Local variables in preemption OS. */
#endif
} jcu_operator_t;
#endif


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/
/* Section: Internal Global */

/******************************************************************************
Private global variables and functions
******************************************************************************/

#if ! R_OSPL_IS_PREEMPTION
static void  R_JCU_OPERATOR_Step1( r_ospl_thread_id_t  in_CurrentThreadID );
static void  R_JCU_OPERATOR_Step2( r_ospl_thread_id_t  in_CurrentThreadID );
static void  R_JCU_OPERATOR_FinalStep1( r_ospl_thread_id_t  in_CurrentThreadID );
static void  R_JCU_OPERATOR_FinalStep2( r_ospl_thread_id_t  in_CurrentThreadID );
#endif
static errnum_t         R_JCU_OPERATOR_DoMessage(void);
static jcu_errorcode_t  R_JCU_OPERATOR_DoMessage_Sub( jcu_context_t*  context );


/***********************************************************************
* Variable: gs_JCU_Operator
************************************************************************/
static jcu_operator_t  gs_JCU_Operator;

/*
* Zero clear means the following initialize.
*    > thread->State = R_OSPL_UNINITIALIZED;
*    > thread->ThreadID = R_OSPL_THREAD_NULL;
*    > thread->MultiThreadEvent = 0;
*    > thread->MultiThreadQueue = NULL;
*    > thread->ExitEvent = 0;
*/


/***********************************************************************
* Variable: gs_JCU_Contexts
************************************************************************/
static jcu_contexts_t  gs_JCU_Contexts;


/***********************************************************************
* Function: R_JCU_Fork
*    Changes current thread to context attached thread.
*
* Arguments:
*    in_out_Context    - <jcu_context_t>
*    in_NewOwnerThread - <r_ospl_thread_id_t>
*    in_NullConfig     - NULL (Reserved)
*
* Return Value:
*    <jcu_errorcode_t> type.
************************************************************************/
jcu_errorcode_t  R_JCU_Fork( jcu_context_t**  in_out_Context,  r_ospl_thread_id_t  in_NewOwnerThread,
                             void*  in_NullConfig )
{
    errnum_t  e, ee;
    jcu_errorcode_t  ej;
    jcu_context_t*   context = *in_out_Context;

    r_ospl_queue_id_t  signleton_lock_ID;
    void*              signleton_lock = NULL;


    signleton_lock_ID = R_OSPL_GetQueueAsSingletonLock();
    e= R_OSPL_QUEUE_Allocate( signleton_lock_ID,  &signleton_lock,  R_OSPL_INFINITE );  /* Lock */
    IF(e)
    {
        goto fin;
    }

    if ( R_JCU_GetOperatorThreadId() == R_OSPL_THREAD_NULL )
    {
        e= R_JCU_CreateOperatorThread();
        IF(e)
        {
            goto fin;
        }
    }

    if ( context == NULL )
    {
        e= R_JCU_OnAllocateContext( &context );
        IF(e)
        {
            goto fin;
        }
        *in_out_Context = context;
    }

    if ( in_NewOwnerThread != R_OSPL_THREAD_NULL )
    {
        enum {         not_set_ID = -1 };
        int_fast32_t   index = not_set_ID;
        jcu_thread_t*  thread;

        e= R_OSPL_TABLE_GetIndex( gs_JCU_Contexts.ThreadMemory,  in_NewOwnerThread,
                                  &index,  R_OSPL_OUTPUT_IF_NOT );
        IF(e)
        {
            goto fin;
        }
        ASSERT_R( index != not_set_ID,  e=E_ACCESS_DENIED; goto fin );
        /* Error: Already attached to a target thread */

        thread = &gs_JCU_Contexts.ThreadMemoryArray[ index ];
        thread->Context = context;
    }

    e=0;
fin:
    if ( signleton_lock != NULL )
    {
        ee= R_OSPL_QUEUE_Free( signleton_lock_ID,  signleton_lock );  /* Unlock */
        e= R_OSPL_MergeErrNum( e, ee );
    }
    if ( e == 0 )
    {
        ej = JCU_ERROR_OK;
    }
    else
    {
        ej = JCU_ERROR_PARAM;
    }
    return  ej;
}


/***********************************************************************
* Function: R_JCU_Join
*    Changes current thread to not context attached thread.
*
* Arguments:
*    in_out_Context    - <jcu_context_t>**
*    in_IsFreeContext  - bool_t
*    in_OldOwnerThread - <r_ospl_thread_id_t>
*
* Return Value:
*    <jcu_errorcode_t> type.
************************************************************************/
jcu_errorcode_t  R_JCU_Join( jcu_context_t**  in_out_Context,  bool_t  in_IsFreeContext,
                             r_ospl_thread_id_t  in_OldOwnerThread )
{
    errnum_t         e, ee;
    jcu_errorcode_t  ej;

    r_ospl_queue_id_t             signleton_lock_ID;
    void*                         signleton_lock = NULL;
    const r_ospl_table_status_t*  table_status;


    signleton_lock_ID = R_OSPL_GetQueueAsSingletonLock();
    e= R_OSPL_QUEUE_Allocate( signleton_lock_ID,  &signleton_lock,  R_OSPL_INFINITE );  /* Lock */
    IF(e)
    {
        goto fin;
    }

    e=0;

    if ( in_OldOwnerThread != R_OSPL_THREAD_NULL )
    {
        R_OSPL_TABLE_Free( gs_JCU_Contexts.ThreadMemory,  in_OldOwnerThread );
    }

    if ( in_IsFreeContext )
    {
        ee= R_JCU_OnFreeContext( in_out_Context );
        e= R_OSPL_MergeErrNum( e, ee );
    }

    ee= R_OSPL_TABLE_GetStatus( gs_JCU_Contexts.ThreadMemory,  &table_status );
    e= R_OSPL_MergeErrNum( e, ee );
    if ( ee == 0  &&  table_status->Count == 0 )
    {
        ee= R_JCU_DestroyOperatorThread();
        e= R_OSPL_MergeErrNum( e, ee );
    }

fin:
    if ( signleton_lock != NULL )
    {
        ee= R_OSPL_QUEUE_Free( signleton_lock_ID,  signleton_lock );  /* Unlock */
        e= R_OSPL_MergeErrNum( e, ee );
    }

    if ( e == 0 )
    {
        ej = JCU_ERROR_OK;
    }
    else
    {
        ej = JCU_ERROR_PARAM;
    }
    return  ej;
}


/***********************************************************************
* Function: R_JCU_StartAsyncAsMessage
*
* Arguments:
*    None.
*
* Return Value:
*    Error Code. 0=No Error.
*
* Description:
*    This function is called from NOT operator thread.
************************************************************************/
errnum_t  R_JCU_StartAsyncAsMessage( r_ospl_async_t* const  ref_Async,
                                     r_ospl_thread_id_t  in_CurrentThread,
                                     r_ospl_thread_id_t  in_OperatorThread )
{
    errnum_t            e;
    jcu_operator_t*     operator_ = &gs_JCU_Operator;
    jcu_message_t*      message = NULL;
    uint32_t            time_out;
    jcu_context_t*      context = R_JCU_GetContext( in_CurrentThread );

    if ( in_CurrentThread == R_OSPL_THREAD_NULL )   /* interrupt */
    {
        time_out = 0u;
    }
    else
    {
        time_out = R_OSPL_INFINITE;
    }

    context->Flags |= 1u << F_JCU_MT_Start;
    ref_Async->ReturnValue = 0;

    e= R_OSPL_QUEUE_Allocate( operator_->MultiThreadQueue,  &message,  time_out );
    IF(e)
    {
        goto fin;
    }
    IF ( context->StatusOfContext == JCU_STATUS_RUN  ||  message == NULL )
    {
        context->CountOfReject += 1;
#if  JCU_DEBUG_MESSAGE_SYNC
        R_D_AddToIntLog( 0x5200000E );
        R_D_AddToIntLog( (int_fast32_t) in_CurrentThread );
        R_D_AddToIntLog( context->StatusOfContext );
        R_D_AddToIntLog( (int_fast32_t) message );
#endif
        if ( message != NULL )
        {
            R_OSPL_QUEUE_Free( operator_->MultiThreadQueue,  message );
        }
        e = E_FIFO_OVER;
        goto fin;
    }
    message->Context = context;
    message->Async = ref_Async;
    context->StatusOfContext = JCU_STATUS_RUN;
    context->Async = ref_Async;
#if  JCU_DEBUG_MESSAGE_SYNC
    R_D_AddToIntLog( 0x52000001 );
    R_D_AddToIntLog( (int_fast32_t) in_CurrentThread );
    R_D_AddToIntLog( ref_Async->A_EventValue );
#endif
    e= R_OSPL_QUEUE_Put( operator_->MultiThreadQueue,  message );
    IF(e)
    {
        goto fin;
    }
    R_OSPL_EVENT_Set( in_OperatorThread,  operator_->MultiThreadEvent );

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_JCU_ContinueAsyncAsMessage
*
* Arguments:
*    None.
*
* Return Value:
*    Error Code. 0=No Error.
*
* Description:
*    This function is called from NOT operator thread.
************************************************************************/
errnum_t  R_JCU_ContinueAsyncAsMessage( r_ospl_async_t* const  ref_Async,
                                        r_ospl_thread_id_t  in_CurrentThread,
                                        r_ospl_thread_id_t  in_OperatorThread,
                                        const jcu_continue_type_t  in_Type )
{
    errnum_t            e;
    jcu_operator_t*     operator_ = &gs_JCU_Operator;
    jcu_message_t*      message = NULL;
    uint32_t            time_out;
    jcu_context_t*      context = R_JCU_GetContext( in_CurrentThread );

    if ( in_CurrentThread == R_OSPL_THREAD_NULL )   /* interrupt */
    {
        time_out = 0u;
    }
    else
    {
        time_out = R_OSPL_INFINITE;
    }

    context->Flags |= 1u << F_JCU_MT_Continue;
    context->ContinueType = in_Type;
    ref_Async->ReturnValue = 0;

    e= R_OSPL_QUEUE_Allocate( operator_->MultiThreadQueue,  &message,  time_out );
    IF(e)
    {
        goto fin;
    }
    IF ( context->StatusOfContext == JCU_STATUS_RUN  ||  message == NULL )
    {
        context->CountOfReject += 1;
#if  JCU_DEBUG_MESSAGE_SYNC
        R_D_AddToIntLog( 0x5200001E );
        R_D_AddToIntLog( (int_fast32_t) in_CurrentThread );
        R_D_AddToIntLog( context->StatusOfContext );
        R_D_AddToIntLog( (int_fast32_t) message );
#endif
        if ( message != NULL )
        {
            R_OSPL_QUEUE_Free( operator_->MultiThreadQueue,  message );
        }
        e = E_FIFO_OVER;
        goto fin;
    }
    message->Context = context;
    message->Async = ref_Async;
    context->StatusOfContext = JCU_STATUS_RUN;
#if  JCU_DEBUG_MESSAGE_SYNC
    R_D_AddToIntLog( 0x52000011 );
    R_D_AddToIntLog( (int_fast32_t) in_CurrentThread );
    R_D_AddToIntLog( ref_Async->A_EventValue );
#endif
    e= R_OSPL_QUEUE_Put( operator_->MultiThreadQueue,  message );
    IF(e)
    {
        goto fin;
    }
    R_OSPL_EVENT_Set( in_OperatorThread,  operator_->MultiThreadEvent );

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_JCU_OPERATOR_FinalizeStart
*
* Arguments:
*    None.
*
* Return Value:
*    Error Code. 0=No Error.
*
* Description:
*    This function is called from NOT operator thread.
*    "jcu_operator_config_t::ReadyOrFinishedEvent" will be set after
*    calling this function.
************************************************************************/
errnum_t  R_JCU_OPERATOR_FinalizeStart( r_ospl_async_t* ref_Async )
{
    r_ospl_thread_id_t  operator_thread = R_JCU_GetOperatorThreadId();

    if ( operator_thread != R_OSPL_THREAD_NULL )
    {
        jcu_operator_t*  operator_context = &gs_JCU_Operator;

        R_OSPL_EVENT_Set( operator_thread,  operator_context->ExitEvent );
    }

    return  0;
}


/***********************************************************************
* Function: R_JCU_OPERATOR_Step0
************************************************************************/
#if ! R_OSPL_IS_PREEMPTION
void  R_JCU_OPERATOR_Step0( r_ospl_thread_id_t  in_CurrentThreadID )
{
    jcu_operator_t*  thread_context = &gs_JCU_Operator;
    thread_context->StepIndex = 1;
}
#endif


/***********************************************************************
* Function: R_JCU_OPERATOR_Dispatch
************************************************************************/
#if ! R_OSPL_IS_PREEMPTION
errnum_t  R_JCU_OPERATOR_Dispatch( r_ospl_thread_id_t  in_ThreadID )
{
    jcu_operator_t*     thread_context = &gs_JCU_Operator;
    r_ospl_thread_id_t  current_thread = R_OSPL_THREAD_GetCurrentId();

    R_OSPL_THREAD_SetCurrentId( in_ThreadID );

    switch ( thread_context->StepIndex )
    {
    case  1:
        R_JCU_OPERATOR_Step1( in_ThreadID );
        break;
    case  2:
        R_JCU_OPERATOR_Step2( in_ThreadID );
        break;
    case  91:
        R_JCU_OPERATOR_FinalStep1( in_ThreadID );
        break;
    case  92:
        R_JCU_OPERATOR_FinalStep2( in_ThreadID );
        break;
    default:
        R_NOOP();
        break;  /* NOT REACHED */
    }

    R_OSPL_THREAD_SetCurrentId( current_thread );
    return  thread_context->Config->ReturnValue;
}
#endif


/***********************************************************************
* Function: R_JCU_OPERATOR_Thread
************************************************************************/
/***********************************************************************
* Function: R_JCU_OPERATOR_Step1
*    Operations of JCU operator thread.
*
* Arguments:
*    None.
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
typedef struct _T_JCU_ThreadConfigClass  T_JCU_ThreadConfigClass;
struct _T_JCU_ThreadConfigClass
{
    errnum_t        ReturnValue;
    r_ospl_async_t  FinishedEvent;

    int_fast32_t     ThreadIndex;
    r_ospl_error_t*  Error;

    const void*  SourceJPEG;
    int_fast32_t             DestinationOffset;
    void*                    JCU_Config;
    int_fast32_t             CodecIndex;
};
#if R_OSPL_IS_PREEMPTION
void  R_JCU_OPERATOR_Thread( void* in_Argument )
#else
static void  R_JCU_OPERATOR_Step1( r_ospl_thread_id_t  in_CurrentThreadID )
#endif
{
#if R_OSPL_IS_PREEMPTION
    jcu_operator_config_t*  thread_config = (jcu_operator_config_t*) in_Argument;
#else
    jcu_operator_config_t*  thread_config = (jcu_operator_config_t*) R_OSPL_THREAD_GetArgument();
#endif
    errnum_t         e;
    jcu_operator_t*  operator_context = &gs_JCU_Operator;
    jcu_errorcode_t  jcu_error;
#if R_OSPL_IS_PREEMPTION
    errnum_t  ee;
    bit_flags32_t  got_flags;
#endif

    static const bit_flags_fast32_t  necessary_flags =
    {
        F_JCU_MultiThreadQueue | F_JCU_ReadyOrFinishedEvent
    };


    ASSERT_R( IS_ALL_BITS_SET( thread_config->Flags,  necessary_flags ),
              e=E_OTHERS; goto fin );

    operator_context->MultiThreadQueue = thread_config->MultiThreadQueue;
    operator_context->State = R_OSPL_RUNNING;

    /*========================= Start of pseudo multithreading */
#if ! R_OSPL_IS_PREEMPTION
    e= R_OSPL_THREAD_SetOnWait( R_OSPL_WAIT_PM_THREAD );
    IF(e!=0)
    {
        goto fin;
    }
#endif
    /*=========================== End of pseudo multithreading */

    e= R_OSPL_EVENT_Allocate(
           &operator_context->ThreadID,  R_OSPL_THREAD_GetCurrentId(),
           &operator_context->MultiThreadEvent,  R_OSPL_UNUSED_FLAG );
    IF(e)
    {
        goto fin;
    }

    e= R_OSPL_EVENT_Allocate(
           &operator_context->ThreadID,  R_OSPL_THREAD_GetCurrentId(),
           &operator_context->ExitEvent,  R_OSPL_UNUSED_FLAG );
    IF(e)
    {
        goto fin;
    }

    operator_context->State = R_OSPL_RUNNABLE;
    operator_context->Config = thread_config;

    R_OSPL_EVENT_Set(
        thread_config->ReadyOrFinishedEvent.A_Thread,
        thread_config->ReadyOrFinishedEvent.A_EventValue );

    jcu_error = R_JCU_Initialize( NULL );
    IF( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }


    /*========================= Start of pseudo multithreading */
#if ! R_OSPL_IS_PREEMPTION
    operator_context->StepIndex = 2;
    e=0;
fin:
    if ( e != 0 )
    {
        operator_context->StepIndex = 91;
    }
    thread_config->ReturnValue = e;
}

/***********************************************************************
* Function: R_JCU_OPERATOR_Step2
************************************************************************/
static void  R_JCU_OPERATOR_Step2( r_ospl_thread_id_t  in_CurrentThreadID )
{
    jcu_operator_t*  operator_context = &gs_JCU_Operator;
    jcu_operator_config_t*  thread_config = operator_context->Config;
    errnum_t       e;
    bit_flags32_t  got_flags;
#else
    /*=========================== End of pseudo multithreading */
    for (;;)
    {
#endif

#if  JCU_DEBUG_MESSAGE_SYNC
    R_D_AddToIntLog( 0x520000A0 );
#endif


    e= R_OSPL_EVENT_Wait( R_OSPL_ANY_FLAG,  &got_flags,  R_OSPL_INFINITE_PSEUDO );
    if ( R_OSPL_THREAD_GetIsWaiting() )
    {
        e=0;
        goto fin;
    }
#if  JCU_DEBUG_MESSAGE_SYNC
    R_D_AddToIntLog( 0x520000A3 );
#endif
    IF(e!=0)
    {
        goto fin;
    }

    operator_context->State = R_OSPL_RUNNING;


    if ( IS_BIT_SET( got_flags,  operator_context->MultiThreadEvent ) )
    {
        e= R_JCU_OPERATOR_DoMessage();
        IF(e)
        {
            goto fin;
        }
    }
    if ( IS_BIT_SET( got_flags,  operator_context->ExitEvent ) )
    {
#if ! R_OSPL_IS_PREEMPTION
        operator_context->StepIndex = 91;
        thread_config->ReturnValue = 0;
        goto fin;
#else
        break;
#endif
    }

    operator_context->State = R_OSPL_RUNNABLE;


    /*========================= Start of pseudo multithreading */
#if ! R_OSPL_IS_PREEMPTION
    operator_context->StepIndex = 2;  /* Loop */
    e=0;
fin:
    if ( e != 0 )
    {
        operator_context->StepIndex = 91;
    }
    thread_config->ReturnValue = e;
}

/***********************************************************************
* Function: R_JCU_OPERATOR_FinalStep1
************************************************************************/
static void  R_JCU_OPERATOR_FinalStep1( r_ospl_thread_id_t  in_CurrentThreadID )
{
    jcu_operator_t*  operator_context = &gs_JCU_Operator;
    jcu_operator_config_t*  thread_config = operator_context->Config;
    errnum_t  e = thread_config->ReturnValue;
    jcu_errorcode_t  jcu_error;
#else
    /*=========================== End of pseudo multithreading */
    }

    e=0;
fin:
#endif


#if  R_OSPL_IS_PREEMPTION

    jcu_error = R_JCU_Terminate();
    IF( jcu_error != JCU_ERROR_OK )
    {
        e= R_OSPL_MergeErrNum( e, E_OTHERS );
    }

    /*========================= Start of pseudo multithreading */
#else
    operator_context->FinalAsync.Flags = R_F_OSPL_A_Thread | R_F_OSPL_A_EventValue;
    operator_context->FinalAsync.A_Thread     = operator_context->ThreadID;
    operator_context->FinalAsync.A_EventValue = operator_context->ExitEvent;

    jcu_error = R_JCU_TerminateAsync( &operator_context->FinalAsync );
    IF( jcu_error != JCU_ERROR_OK )
    {
        e= R_OSPL_MergeErrNum( e, E_OTHERS );
    }

    operator_context->StepIndex = 92;
    thread_config->ReturnValue = e;
}

/***********************************************************************
* Function: R_JCU_OPERATOR_FinalStep2
************************************************************************/
static void  R_JCU_OPERATOR_FinalStep2( r_ospl_thread_id_t  in_CurrentThreadID )
{
    jcu_operator_t*  operator_context = &gs_JCU_Operator;
    jcu_operator_config_t*  thread_config = operator_context->Config;
    errnum_t  e = thread_config->ReturnValue;
    errnum_t  ee;


    ee= R_OSPL_EVENT_Wait( operator_context->FinalAsync.A_EventValue,
                           NULL,  R_OSPL_INFINITE_PSEUDO );
    if ( R_OSPL_THREAD_GetIsWaiting() )
    {
        goto fin;
    }
    e= R_OSPL_MergeErrNum( e, ee );

#endif
    /*=========================== End of pseudo multithreading */


    ee= R_OSPL_EVENT_Free( &operator_context->ThreadID, &operator_context->ExitEvent );
    e= R_OSPL_MergeErrNum( e, ee );

    ee= R_OSPL_EVENT_Free( &operator_context->ThreadID, &operator_context->MultiThreadEvent );
    e= R_OSPL_MergeErrNum( e, ee );

    R_OSPL_NOTIFY_ERROR( /*Set*/ &thread_config->ReturnValue,  e );
    operator_context->ThreadID = R_OSPL_THREAD_NULL;
    operator_context->State = R_OSPL_UNINITIALIZED;


    /*========================= Start of pseudo multithreading */
#if ! R_OSPL_IS_PREEMPTION
    operator_context->StepIndex = 0;
#endif
    /*=========================== End of pseudo multithreading */

    R_OSPL_EVENT_Set(
        thread_config->ReadyOrFinishedEvent.A_Thread,
        thread_config->ReadyOrFinishedEvent.A_EventValue );
#if ! R_OSPL_IS_PREEMPTION
fin:
    ;
#endif
    R_DEBUG_BREAK_IF_ERROR();
    R_OSPL_FreeCurrentThreadError();
#if  defined( USE_LIST_FREE_RTOS )
    vTaskDelete( NULL );
#endif
}


/***********************************************************************
* Function: R_JCU_OPERATOR_DoMessage
*    R_JCU_OPERATOR_DoMessage.
*
* Arguments:
*    ref_F_Context - <jcu_context_t>
*
* Return Value:
*    Error code.  If there is no error, the return value is 0
************************************************************************/
static errnum_t  R_JCU_OPERATOR_DoMessage()
{
    errnum_t         e, ee;
    jcu_errorcode_t  ej;
    jcu_context_t*   context;
    jcu_message_t*   message = NULL;

    jcu_operator_t* const  operator_context = &gs_JCU_Operator;


    for (;;)
    {
#if  JCU_DEBUG_MESSAGE_SYNC
        R_D_AddToIntLog( 0x520000A4 );
#endif

        e= R_OSPL_QUEUE_Get( operator_context->MultiThreadQueue,  &message,  0 );
        IF(e)
        {
            goto fin;
        }
        if ( message == NULL )
        {
            break;
        }
        context = message->Context;
#if  JCU_DEBUG_MESSAGE_SYNC
        R_D_AddToIntLog( 0x520000A5 );
        R_D_AddToIntLog( (int_fast32_t) context->OwnerThread );
#endif

        ej= R_JCU_OPERATOR_DoMessage_Sub( context );

        context->StatusOfContext = JCU_STATUS_READY;
        R_OSPL_NOTIFY_ERROR( /*Set*/ &message->Async->ReturnValue,  (errnum_t) ej );
#if  JCU_DEBUG_MESSAGE_SYNC
        R_D_AddToIntLog( 0x520000A6 );
        R_D_AddToIntLog( (int_fast32_t) message->Async->A_Thread );
        R_D_AddToIntLog( message->Async->A_EventValue );
#endif
        R_OSPL_EVENT_Set( message->Async->A_Thread,  message->Async->A_EventValue );

        e= R_OSPL_QUEUE_Free( operator_context->MultiThreadQueue,  message );
        message = NULL;
        IF(e)
        {
            goto fin;
        }
    }
    e=0;
fin:
    if ( message != NULL )
    {
        context->StatusOfContext = JCU_STATUS_READY;
        ee= R_OSPL_QUEUE_Free( operator_context->MultiThreadQueue,  message );
        e= R_OSPL_MergeErrNum( e, ee );
    }
    return  e;
}


/***********************************************************************
* Function: R_JCU_OPERATOR_DoMessage_Sub
*    R_JCU_OPERATOR_DoMessage.
*
* Arguments:
*    ref_F_Context - <jcu_context_t>
*
* Return Value:
*    Error code.  If there is no error, the return value is 0
************************************************************************/
static jcu_errorcode_t  R_JCU_OPERATOR_DoMessage_Sub( jcu_context_t*  context )
{
    jcu_errorcode_t  ej;
    int_fast32_t     index;


    if ( IS_BIT_SET( context->Flags,  1u << F_JCU_MT_Codec )  ||
            gs_JCU_Operator.Owner[ F_JCU_MT_Codec ] != context )
    {
        context->Flags &= ~( 1u << F_JCU_MT_Codec );
        gs_JCU_Operator.Owner[ F_JCU_MT_Codec ] = context;

        ej= R_JCU_SelectCodec( context->Codec );
        IF ( ej != JCU_ERROR_OK )
        {
            goto fin;
        }

        /* For changing from JCU_STATUS_SELECTED to JCU_STATUS_READY. */
        if ( context->Codec == JCU_DECODE )
        {
            context->Flags |= ( 1u << F_JCU_MT_DecodeAndBuffer );
        }
        else
        {
            context->Flags |= ( 1u << F_JCU_MT_EncodeAndBuffer );
        }
    }


    /* Watch encode tables for T_JCU_Multithread_Context test. */
#if  ! GS_IS_ALWAYS_SET_ENCODE_TABLES
    {
        bool_t  was_all_enabled = R_OSPL_DisableAllInterrupt();

        if ( JCU.JCHTBA0 != 0x00 )
        {
            R_DEBUG_BREAK();    /* Not break */
        }
        if ( JCU.JCHTBA0 != 0x00 )
        {
            R_DEBUG_BREAK();    /* Break */
        }

        if ( was_all_enabled )
        {
            R_OSPL_EnableAllInterrupt();
        }
    }
#endif


    if ( context->Codec == JCU_DECODE )
    {
        if ( IS_BIT_SET( context->Flags,  1u << F_JCU_MT_DecodeAndBuffer )  ||
                gs_JCU_Operator.Owner[ F_JCU_MT_DecodeAndBuffer ] != context )
        {
            context->Flags &= ~( 1u << F_JCU_MT_DecodeAndBuffer );
            gs_JCU_Operator.Owner[ F_JCU_MT_DecodeAndBuffer ] = context;

            ej= R_JCU_SetDecodeParam( &context->Decode,  &context->Buffer );
            IF ( ej != JCU_ERROR_OK )
            {
                goto fin;
            }
        }

        if ( IS_BIT_SET( context->Flags,  1u << F_JCU_MT_IsPauseForImageInfo )  ||
                gs_JCU_Operator.Owner[ F_JCU_MT_IsPauseForImageInfo ] != context )
        {
            context->Flags &= ~( 1u << F_JCU_MT_IsPauseForImageInfo );
            gs_JCU_Operator.Owner[ F_JCU_MT_IsPauseForImageInfo ] = context;

            ej= R_JCU_SetPauseForImageInfo( context->IsPauseForImageInfo );
            IF ( ej != JCU_ERROR_OK )
            {
                goto fin;
            }
        }
    }


    if ( context->Codec == JCU_ENCODE )
    {
        if ( IS_BIT_SET( context->Flags,  1u << F_JCU_MT_EncodeAndBuffer )  ||
                gs_JCU_Operator.Owner[ F_JCU_MT_EncodeAndBuffer ] != context )
        {
            context->Flags &= ~( 1u << F_JCU_MT_EncodeAndBuffer );
            gs_JCU_Operator.Owner[ F_JCU_MT_EncodeAndBuffer ] = context;

            ej= R_JCU_SetEncodeParam( &context->Encode,  &context->Buffer );
            IF ( ej != JCU_ERROR_OK )
            {
                goto fin;
            }
        }

        for ( index = 0;  index < R_COUNT_OF( context->QuantizationTable );  index += 1 )
        {
            if ( IS_BIT_SET( context->Flags,  1u << ( F_JCU_MT_QuantizationTable + index ) )  ||
                    gs_JCU_Operator.Owner[ F_JCU_MT_QuantizationTable + index ] != context )
            {
#if  ! GS_IS_ALWAYS_SET_ENCODE_TABLES
                context->Flags &= ~( 1u << ( F_JCU_MT_QuantizationTable + index ) );
#endif
                gs_JCU_Operator.Owner[ F_JCU_MT_QuantizationTable + index ] = context;

                ej= R_JCU_SetQuantizationTable( (jcu_table_no_t) index,
                                                context->QuantizationTable[ index ] );
                IF ( ej != JCU_ERROR_OK )
                {
                    goto fin;
                }
            }
        }

        for ( index = 0;  index < R_COUNT_OF( context->HuffmanTableAC );  index += 1 )
        {
            int_fast32_t   bit_index = F_JCU_MT_HuffmanTable + index * 2 + JCU_HUFFMAN_AC;
            uint_fast32_t  bit = 1u << bit_index;

            if ( IS_BIT_SET( context->Flags,  bit )  ||
                    gs_JCU_Operator.Owner[ bit_index ] != context )
            {
#if  ! GS_IS_ALWAYS_SET_ENCODE_TABLES
                context->Flags &= ~bit;
#endif
                gs_JCU_Operator.Owner[ bit_index ] = context;

                ej= R_JCU_SetHuffmanTable( (jcu_table_no_t) index,  JCU_HUFFMAN_AC,
                                           context->HuffmanTableAC[ index ] );
                IF ( ej != JCU_ERROR_OK )
                {
                    goto fin;
                }
            }
        }

        for ( index = 0;  index < R_COUNT_OF( context->HuffmanTableDC );  index += 1 )
        {
            int_fast32_t   bit_index = F_JCU_MT_HuffmanTable + index * 2 + JCU_HUFFMAN_DC;
            uint_fast32_t  bit = 1u << bit_index;

            if ( IS_BIT_SET( context->Flags,  bit )  ||
                    gs_JCU_Operator.Owner[ bit_index ] != context )
            {
#if  ! GS_IS_ALWAYS_SET_ENCODE_TABLES
                context->Flags &= ~bit;
#endif
                gs_JCU_Operator.Owner[ bit_index ] = context;

                ej= R_JCU_SetHuffmanTable( (jcu_table_no_t) index,  JCU_HUFFMAN_DC,
                                           context->HuffmanTableDC[ index ] );
                IF ( ej != JCU_ERROR_OK )
                {
                    goto fin;
                }
            }
        }
    }


    if ( IS_BIT_SET( context->Flags,  1u << F_JCU_MT_ErrorFilter )  ||
            gs_JCU_Operator.Owner[ F_JCU_MT_ErrorFilter ] != context )
    {
        context->Flags &= ~( 1u << F_JCU_MT_ErrorFilter );
        gs_JCU_Operator.Owner[ F_JCU_MT_ErrorFilter ] = context;

        ej= R_JCU_SetErrorFilter( context->ErrorFilter );
        IF ( ej != JCU_ERROR_OK )
        {
            goto fin;
        }
    }

#if  JCU_IS_COUNT_MODE
    if ( IS_BIT_SET( context->Flags,  1u << F_JCU_MT_CountMode )  ||
            gs_JCU_Operator.Owner[ F_JCU_MT_CountMode ] != context )
    {
        context->Flags &= ~( 1u << F_JCU_MT_CountMode );
        gs_JCU_Operator.Owner[ F_JCU_MT_CountMode ] = context;

        ej= R_JCU_SetCountMode( &context->CountMode );
        IF ( ej != JCU_ERROR_OK )
        {
            goto fin;
        }
    }
#endif


    if ( IS_ANY_BITS_SET( context->Flags,  ( 1u << F_JCU_MT_Start ) | ( 1u << F_JCU_MT_Continue ) ) )
    {

        if ( IS_BIT_SET( context->Flags,  1u << F_JCU_MT_Start ) )
        {
            context->Flags &= ~( 1u << F_JCU_MT_Start );

            ej= R_JCU_Start();
            IF ( ej != JCU_ERROR_OK )
            {
                goto fin;
            }

            if ( context->IsPauseForImageInfo )
            {

                ej= R_JCU_GetImageInfo( &context->ImageInfo );
                IF ( ej != JCU_ERROR_OK )
                {
                    goto fin;
                }
            }
        }

        if ( IS_BIT_SET( context->Flags,  1u << F_JCU_MT_Continue ) )
        {
            context->Flags &= ~( 1u << F_JCU_MT_Continue );

            ej= R_JCU_Continue( context->ContinueType );
            IF ( ej != JCU_ERROR_OK )
            {
                goto fin;
            }
        }

        if ( context->Codec == JCU_ENCODE )
        {
            const jcu_async_status_t*  status;

            R_JCU_GetAsyncStatus( &status );
            if ( ! status->IsPaused )
            {

                ej= R_JCU_GetEncodedSize( &context->EncodedSize );
                IF( ej != JCU_ERROR_OK )
                {
                    goto fin;
                }
            }
        }
    }

    ej = JCU_ERROR_OK;
fin:
    return  ej;
}


/***********************************************************************
* Function: R_JCU_GetOperatorThreadId
*    Gets operator thread.
*
* Arguments:
*    None.
*
* Return Value:
*    operator thread.
************************************************************************/
r_ospl_thread_id_t  R_JCU_GetOperatorThreadId(void)
{
    return  gs_JCU_Operator.ThreadID;
}


/***********************************************************************
* Function: R_JCU_GetOperatorConfig
*    Gets a context of operator thread.
*
* Arguments:
*    None.
*
* Return Value:
*    operator thread.
************************************************************************/
jcu_operator_config_t*  R_JCU_GetOperatorConfig(void)
{
    return  gs_JCU_Operator.Config;
}


/***********************************************************************
* Function: R_JCU_GetContext
*    Gets context of specified thread.
*
* Arguments:
*    in_Thread - current thread, R_OSPL_THREAD_NULL = interrupt
*
* Return Value:
*    Context of specified thread, context of current interrupt or NULL.
*
* Description:
*    In interrupt context, call <R_JCU_SetContextInInterrupt>
*    before this function is called.
************************************************************************/
jcu_context_t*  R_JCU_GetContext( r_ospl_thread_id_t  in_Thread )
{
    errnum_t        e;
    jcu_context_t*  context = NULL;

    if ( R_JCU_GetOperatorThreadId() == R_OSPL_THREAD_NULL )
    {
        goto fin;
    }

    if ( in_Thread != R_OSPL_THREAD_NULL )
    {
        enum {          not_set_ID = -1 };
        int_fast32_t    index = not_set_ID;

        e= R_OSPL_TABLE_GetIndex( gs_JCU_Contexts.ThreadMemory,  in_Thread,
                                  &index,  R_OSPL_DO_NOTHING_IF_NOT );
        IF(e)
        {
            goto fin;
        }
        if ( index != not_set_ID )
        {
            context = gs_JCU_Contexts.ThreadMemoryArray[ index ].Context;
        }
    }
    else
    {
        context = gs_JCU_Contexts.CurrentContextInInterrupt;
        ASSERT_R( context != NULL  ||  R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL,
                  R_OSPL_RaiseUnrecoverable( E_OTHERS ) );
        /* Error: Call "R_JCU_SetContextInInterrupt" before calling from interrupt.. */
    }
fin:
    return  context;
}


/***********************************************************************
* Function: R_JCU_SetContextInInterrupt
*    Sets F-context of current interrupt.
*
* Arguments:
*    in_NewCurrentContext - <jcu_context_t>
*
* Return Value:
*    Previous F-context of interrupt.
*
* Example:
*    > void  OnInterrupt()
*    > {
*    >     jcu_config_t*   interrupt_JCU_config = ...;
*    >     jcu_context_t*  this_level_context = interrupt_JCU_config->OwnerContext;
*    >     jcu_context_t*  low_level_context = R_JCU_SetContextInInterrupt( this_level_context );
*    >
*    >     R_JCU_Method();
*    >
*    >     R_JCU_SetContextInInterrupt( low_level_context );
*    > }
************************************************************************/
jcu_context_t*  R_JCU_SetContextInInterrupt( jcu_context_t*  in_NewCurrentContext )
{
    jcu_context_t*  low_level_context = gs_JCU_Contexts.CurrentContextInInterrupt;
    gs_JCU_Contexts.CurrentContextInInterrupt = in_NewCurrentContext;
    return  low_level_context;
}


/***********************************************************************
* Function: R_JCU_CONTEXTS_Initialize
*
* Arguments:
*    in_out_Config - <jcu_operator_config_t>
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_JCU_CONTEXTS_Initialize( jcu_contexts_config_t*  in_out_Config )
{
    errnum_t         e;
    jcu_contexts_t*  contexts = &gs_JCU_Contexts;

    static const bit_flags_fast32_t  necessary_flags =
    {
        F_JCU_ThreadMemory | F_JCU_ThreadMemoryArray
    };

    ASSERT_R( IS_ALL_BITS_SET( in_out_Config->Flags,  necessary_flags ),  e=E_OTHERS; goto fin );

    contexts->ThreadMemory      = in_out_Config->ThreadMemory;
    contexts->ThreadMemoryArray = in_out_Config->ThreadMemoryArray;

    e=0;
fin:
    return  e;
}


#endif  /* JCU_MULTI_THREAD */
